//////////////////////////////////////////////////////////////////////////////////////////
// Code to create a QComboBox where multiple items can be selected and the items
// are shown in a tree.
// Written by Corwin Joy
// This code is hereby placed in the Public Domain.
// This code comes with no warranty of any kind, use at your own risk.qwe c

#include "neumulticombo.h"
#include <QTreeView>
#include <QAbstractItemView>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QTimer>
#include <QKeyEvent>
#include <QHeaderView>
#include <QLineEdit>
#include <QEvent>
#include <QMouseEvent>
#include <QModelIndex>
#include <cassert>

//////////////////////////////////////////////////////////////////////////////////////////

/// NeuMultiCombo

NeuMultiCombo::NeuMultiCombo(int maxCheck, QModelIndex const defaultRootIdx, QWidget* parent) : 
QComboBox(parent), skipNextHide(false), defaultRootIndex(defaultRootIdx) //Widget creation
{
  QTreeView* treeView = new QTreeView(this);
  treeView->header()->hide();
  treeView->setMaximumHeight(500); // by default tree will show as 1 line high, need to reserve room
  treeView->setMinimumHeight(500);
  setView(treeView);
  view()->viewport()->installEventFilter(this);
  view()->setSelectionMode(QAbstractItemView::SingleSelection); // click to toggle items
  setEditable(true);
  lineEdit()->setReadOnly(true);

  QStandardItemModel *model  = new QStandardItemModel;
  setModel(model);
  setRoot(model->indexFromItem(model->invisibleRootItem()));

  setInsertPolicy(QComboBox::NoInsert);
  checkCount = 0;
  checkMax = maxCheck;
}

void NeuMultiCombo::showPopup()
{
  setRootModelIndex(defaultRootIndex);
  QComboBox::showPopup();
}

void NeuMultiCombo::hidePopup()
{
  if (skipNextHide)
    skipNextHide = false;
  else
  {
    UpdateSelectionString(); 
    QComboBox::hidePopup();
    currentIndexChanged(currentIndexes());
    lineEdit()->setText(strSelected);
  }
}

void NeuMultiCombo::addItem(const QString &text, const QVariant &userData)
{
  QStandardItem *item = new QStandardItem(text);
  item->setCheckable(true);
  item->setData(userData);
  model()->invisibleRootItem()->appendRow(item);
}

QStandardItemModel* NeuMultiCombo::model() const
{
  return static_cast<QStandardItemModel *>(QComboBox::model());
}

void NeuMultiCombo::ToggleItem(QModelIndex const &index)
{
  // if this item can be checked, as per Qt::ItemIsUserCheckable
  // check the box if this item is selected
  // (typically, checkboxes are enabled via item->setCheckable(true))
  if(model()->data(index, Qt::CheckStateRole).toInt() == Qt::Checked)
  {
    // uncheck
    QModelIndexList indexes = getChildren(index);
    Q_FOREACH(QModelIndex idx, indexes)
    {
      if(model()->flags(idx) & Qt::ItemIsUserCheckable)
        model()->setData(idx, Qt::Unchecked, Qt::CheckStateRole);
    }
    checkCount--;
  }
  else if (checkCount < checkMax)
  {
    // check
    QModelIndexList indexes = getChildren(index);
    Q_FOREACH(QModelIndex idx, indexes)
    {
      if(model()->flags(idx) & Qt::ItemIsUserCheckable)
        model()->setData(idx, Qt::Checked, Qt::CheckStateRole);
    }
    checkCount++;
  }

  // update the check flags for the parents
  bool ch, uc;
  setCheckState(defaultRootIndex, false, true, ch, uc);
}

// this routine updates the check flags for the tree starting from the current node
// unchecked = all children are unchecked
// checked = all children are checked
// partial = some children are checked
void NeuMultiCombo::setCheckState(QModelIndex current, bool bForceChildrenChecked,
                                 bool bNeverForceChildren,
                                 bool &bChildrenChecked, bool &bChildrenUnChecked)
{
  if(current != QModelIndex())
  {
    if(bForceChildrenChecked)
    {
      // force children to be checked
      model()->setData(current, Qt::Checked, Qt::CheckStateRole);
      bChildrenUnChecked = false;
    }

    // leaf node, just return the state
    if(!model()->hasChildren(current))
    {
      if(model()->data(current, Qt::CheckStateRole).toInt() == Qt::Checked)
      {
        bChildrenChecked = true;
        bChildrenUnChecked = false;
      }
      else
      {
        bChildrenChecked = false;
        bChildrenUnChecked = true;
      }
      return;
    }

    // interior node with children. set checkbox based on children
    bChildrenChecked = false;
    bChildrenUnChecked = false;

    if(!bNeverForceChildren && model()->data(current, Qt::CheckStateRole).toInt() == Qt::Checked)
    {
      // force children to be checked
      bForceChildrenChecked = true;
    }

    int rows = model()->rowCount(current);
    int cols = model()->columnCount(current);
    for(int row = 0; row < rows; ++row)
    {
      for(int col = 0; col < cols; ++col)
      {
        bool ch, un;
        ch = false;
        un = false;
        setCheckState(current.child(row, col), bForceChildrenChecked, bNeverForceChildren, ch, un);
        bChildrenChecked |= ch;
        bChildrenUnChecked |= un;
      }
    }

    if(bChildrenUnChecked && !bChildrenChecked)
    {
      model()->setData(current, Qt::Unchecked, Qt::CheckStateRole);
    }
    else if(!bChildrenUnChecked && bChildrenChecked)
    {
      model()->setData(current, Qt::Checked, Qt::CheckStateRole);
    }
    else if(bChildrenUnChecked && bChildrenChecked)
    {
      model()->setData(current, Qt::PartiallyChecked, Qt::CheckStateRole);
    }
    else // (!bChildrenUnChecked && !bChildrenChecked)
    {
      // can only get here if loop above failed to execute.
      assert(!"Invalid check state encountered for child checkboxes.\n"
        "Can't have no checked and no unchecked children.");
    }
  }
  else
  {
    // Treat this as a search from the (hidden) root.
    // Fortunately, we don't need to worry about the hidden root having a checkbox
    current = model()->index(0, 0, defaultRootIndex);
    int topRow = 0;
    while(current.isValid())
    {
      bForceChildrenChecked = false;
      if(!bNeverForceChildren && model()->data(current, Qt::CheckStateRole).toInt() == Qt::Checked)
      {
        // force children to be checked
        bForceChildrenChecked = true;
      }
      bool ch, un;
      ch = false;
      un = false;
      setCheckState(current, bForceChildrenChecked, bNeverForceChildren, ch, un);
      ++topRow;
      current = model()->index(topRow, 0, defaultRootIndex);
    }
  }

}

// Return a list containing the current item + all its children
QModelIndexList NeuMultiCombo::getChildren(QModelIndex current) const
{
  QModelIndexList indexes;

  if(current != QModelIndex())
  {
    indexes.push_back(current);
    int rows = model()->rowCount(current);
    int cols = model()->columnCount(current);
    for(int row = 0; row < rows; ++row)
    {
      for(int col = 0; col < cols; ++col)
      {
        indexes += getChildren(current.child(row, col));
      }
    }
  }
  else
  {
    // Treat this as a search from the root.
    // We need special handling here because the defaultRootIndex may be
    // the invalid index QModelIndex(), if the defaultRootIndex comes from 
    // QStandardItemModel::invisibleRootItem().
    //
    current = model()->index(0, 0, defaultRootIndex);
    int topRow = 0;
    while(current.isValid())
    {
      indexes += getChildren(current);
      ++topRow;
      current = model()->index(topRow, 0, defaultRootIndex);
    }
  }

  return indexes;
}

// Return a list containing minimal checked representation of the tree
QModelIndexList NeuMultiCombo::getCheckedChildren(QModelIndex current) const
{
  QModelIndexList indexes;

  if(current != QModelIndex())
  {
    if(model()->data(current, Qt::CheckStateRole).toInt() == Qt::Checked)
    {
      indexes.push_back(current);
    }
    
    if(model()->data(current, Qt::CheckStateRole).toInt() != Qt::Checked &&
      model()->data(current, Qt::CheckStateRole).toInt() != Qt::Unchecked)
    {
      // only expand children if this node is not checked or purely unchecked
      int rows = model()->rowCount(current);
      int cols = model()->columnCount(current);
      for(int row = 0; row < rows; ++row)
      {
        for(int col = 0; col < cols; ++col)
        {
          indexes += getCheckedChildren(current.child(row, col));
        }
      }
    }
  }
  else
  {
    // Treat this as a search from the root.
    // We need special handling here because the defaultRootIndex may be
    // the invalid index QModelIndex(), if the defaultRootIndex comes from 
    // QStandardItemModel::invisibleRootItem().
    //
    current = model()->index(0, 0, defaultRootIndex);
    int topRow = 0;
    while(current.isValid())
    {
      indexes += getCheckedChildren(current);
      ++topRow;
      current = model()->index(topRow, 0, defaultRootIndex);
    }
  }

  return indexes;
}

bool NeuMultiCombo::eventFilter(QObject* object, QEvent* event)
{
  if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease)
  {
    QMouseEvent* m = static_cast<QMouseEvent*>(event); 
    QModelIndex index = view()->indexAt(m->pos());
    QRect vrect = view()->visualRect(index);

    if(event->type() == QEvent::MouseButtonPress  && 
      (model()->flags(index) & Qt::ItemIsUserCheckable) &&
      vrect.contains(m->pos()))
    {
      ToggleItem(index);
      UpdateSelectionString(); 
    }
    if (view()->rect().contains(m->pos()))
      skipNextHide = true;
  }
  return QComboBox::eventFilter(object, event);
}

void NeuMultiCombo::UpdateSelectionString()
{
  QModelIndex topidx = model()->index(0, 0, defaultRootIndex);
  if(!topidx.isValid())
    return;

  QModelIndexList idxes = getCheckedChildren(defaultRootIndex);
  strSelected = "";
  Q_FOREACH(QModelIndex idx, idxes)
    strSelected += idx.data().toString() + ",";
  if(strSelected.size() > 0)
    strSelected.truncate(strSelected.size() - 1); // kill trailing ","
}

// get the currently checked indexes
QModelIndexList NeuMultiCombo::currentIndexes() const
{
  QModelIndexList indexes = getChildren(defaultRootIndex);
  QModelIndexList matches;
  Q_FOREACH(QModelIndex index, indexes)
  {
    if(model()->data(index, Qt::CheckStateRole).toInt() == Qt::Checked)
      matches.push_back(index);
  }

  return matches;
}

// set the currently checked indexes
void NeuMultiCombo::setCurrentIndexes(QModelIndexList indexes)
{
  QModelIndexList old = currentIndexes();
  Q_FOREACH(QModelIndex idx, old)
    model()->setData(idx, Qt::Unchecked, Qt::CheckStateRole);
  Q_FOREACH(QModelIndex idx, indexes)
    model()->setData(idx, Qt::Checked, Qt::CheckStateRole);

  // update the check flags for the parents
  bool ch, uc;
  setCheckState(defaultRootIndex, false, false, ch, uc);

  UpdateSelectionString();
  currentIndexChanged(currentIndexes());
}


QVariant NeuMultiCombo::itemData(QModelIndex index, int role /*= Qt::UserRole */) const
{
  return model()->data(index, role);
}

QModelIndexList NeuMultiCombo::findData (const QVariant & data, int role, Qt::MatchFlags flags) const 
{
  QModelIndex start = model()->index(0, 0, defaultRootIndex);
  return model()->match(start, role, data, -1, flags | Qt::MatchRecursive);
}

QModelIndexList NeuMultiCombo::findText (const QString & text, Qt::MatchFlags flags) const 
{
  return findData(text, Qt::DisplayRole , flags);
}
/*
///////////////////////////////////////////////////////////////////////////
// StandardTreeComboBox

StandardTreeComboBox::StandardTreeComboBox(QWidget* parent) : TreeComboBox(QModelIndex(), parent)
{
  QStandardItemModel *model  = new QStandardItemModel;
  setModel(model);
  setRoot(model->indexFromItem(model->invisibleRootItem()));
}

static void sortNonCaseSensitive( QStringList &sList ) {
  ///  Sorts the passed sList non-case-sensitively.
  ///  (Preserves the cases! Just doesn't use them
  ///  while sorting.)
  QMap<QString, QString> strMap;
  Q_FOREACH(QString str, sList) 
  {
    strMap.insert(str.toLower(), str);
  }
  sList = strMap.values();
}

QStandardItemModel *StandardTreeComboBox::model() const
{
  return static_cast<QStandardItemModel *>(TreeComboBox::model());
}

void StandardTreeComboBox::addItem(const QString &text, const QVariant &userData)
{
  QStandardItem *item = new QStandardItem(text);
  item->setCheckable(true);
  item->setData(userData);
  model()->invisibleRootItem()->appendRow(item);
}
*/